/*
** Module   : VT100SwingKeyboard.java
** Abstract : VT-100 Swing keyboard implementation.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------Description-----------------
** 001 MAG 20140318 First version.
** 002 MAG 20140507 Fix RETURN key code from JS on Windows OS.
*/

package com.goldencode.p2j.ui.client.chui.driver.swing;

import java.awt.event.*;

import com.goldencode.terminal.*;
import com.goldencode.util.*;

/**
 * VT-100 Swing keyboard implementation. Also used for JS code.
 * 
 * @author mag
 *
 */
public class VT100SwingKeyboard
extends VT100AbstractKeyboard
implements KeyListener
{  
   /** RETURN key code */
   public static final char RETURN = '\r';

   /** OS platform flag */
   private boolean isWindowsOS = false;
   
   /**
    * Constructor.
    */
   public VT100SwingKeyboard()
   {
      isWindowsOS = PlatformHelper.isUnderWindowsFamily();
   }
   
   /**
    * Key pressed event handler.
    *  
    * @param   event
    *          Keyboard event.
    */
   @Override
   public void keyPressed(KeyEvent event)
   {
      handle(event.getKeyCode(), event.getKeyChar());         
   }
   
   /**
    * Key typed event handler.
    *  
    * @param   event
    *          Keyboard event.
    */
   @Override
   public void keyTyped(KeyEvent event)
   {
      // not used
   }

   /**
    * Key released event handler.
    *  
    * @param   event
    *          Keyboard event.
    */
   @Override
   public void keyReleased(KeyEvent event)
   {      
      // not used
   }

   /**
    * Handle key strokes. VT100AbstractKeyboard class has been designed to emulate
    * a VT100 keyboard and some keystrokes are transformed before sending to terminal. 
    * Windows OS does not use VT100 keyboard codes and should be send unchanged.
    * 
    * @param   keyCode
    *          Key code.
    * @param   charCode
    *          Character code.
    */
   public void handle(int keyCode, char charCode)
   {
      if (isWindowsOS)
      {
         // On Windows OS send keystrokes directly
         switch (keyCode)
         {
            case KeyEvent.VK_CONTROL:
            case KeyEvent.VK_SHIFT:
            case KeyEvent.VK_ALT:
            case KeyEvent.VK_CAPS_LOCK:    
            case KeyEvent.VK_SCROLL_LOCK:
            case KeyEvent.VK_NUM_LOCK:
            case KeyEvent.VK_PAUSE:
            case KeyEvent.VK_WINDOWS:
            case KeyEvent.VK_CONTEXT_MENU:
               break;
            default:
               handleKeystroke(-1, charCode);
               // CR and LF
               if (charCode == RETURN)
               {
                  handleKeystroke(-1, '\n');
               }
               break;
         }
      }
      else
      {
         handleKeystroke(keyCode, charCode);                  
      }
   }
   
   /**
    * Handle JS key codes.
    * 
    * @param   keyCode
    *          Key code.
    * @param   charCode
    *          Character code
    */
   public void injectKey(int keyCode, char charCode)
   {
      handle(keyCode, charCode);
   }
}
